###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.SimpleItem
import Products
from Products.WarpFramework import base_item
import string

manage_add_bookmark_form = HTMLFile('dtml/bookmark_add', globals())


def manage_add_bookmark(self, local_file, url, description, REQUEST=None):
	"Add an instance of the bookmark class"

	new_id = self.get_unique_id()

	if local_file:

		# How should we encode this url?
		# FIXME

		# We should store it in a raw state, so that
		# we can display it in a human-readable form
		# if it is local..

		url = local_file
		is_local_file = 1

	else:

		is_local_file = 0

	self._setObject(new_id, bookmark(new_id, url, description, is_local_file))

	if REQUEST is not None:

		REQUEST.RESPONSE.redirect(self.absolute_url())


class bookmark(
	#Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.SimpleItem.Item,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"""Used to capture bookmarks, either as remote resources or local files."""

	meta_type = 'Bookmark'
	icon = 'misc_/BookmarkFolder/dtmldoc.gif'
	help_name = 'bookmark.stx'

	_properties = (
		{'id':'description', 'type':'text'},
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View Bookmark',
		 ('', 'render_valid_url')),
		)


	index_html = HTMLFile('dtml/bookmark_index', globals())


	def render_valid_url(self):
		"""Returns an <a href=""> valid url."""

		if not self.is_local_file:

			if string.find(self.url, 'https://') > -1 or \
				string.find(self.url, 'http://') > -1:

				return self.url

			else:

				return 'http://' + self.url

		else:

			# FIXME
			# This isn't good enough for non-UNIX systems

			return 'file://' + self.url


	def make_event(self, REQUEST):
		"""Creates an event based on the bookmark."""

		return self.get_workspace().make_event(REQUEST=REQUEST,
				bookmarks=[self.render_valid_url()])


	def make_todo(self, REQUEST):
		"""Creates a todo based on the bookmark."""

		return self.get_workspace().make_todo(REQUEST=REQUEST,
				bookmarks=[self.render_valid_url()])


	def __init__(self, id, url, description, is_local_file):
		"Initialize the class instance"

		self.id = id
		self.url = url
		self.description = description
		self.is_local_file = is_local_file


default__class_init__(bookmark)