###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@thingamy.net>
#   Copyright (c) 2000,2001 Thingamy Ltd.
#   Copyright (c) 2001 James E. Blair <corvus@gnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.17 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import DTMLFile
from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
from DateTime import DateTime
import Acquisition
import AccessControl.Role
import OFS.Folder
from Products.ZCatalog import CatalogAwareness
import string
from Products.WarpFramework import base_item

expand_hour = lambda x: x * 60 * 60
expand_minute = lambda x: x * 60

manage_add_calendar_event_form = DTMLFile('dtml/calendar_event_add', globals())

def manage_add_calendar_event(self, REQUEST=None, RESPONSE=None):
	"""Add an instance of the calendar_event class."""

	new_id = str(self.get_unique_id())

	self._setObject(new_id, calendar_event(new_id))

	if REQUEST is not None:

		RESPONSE.redirect(self.absolute_url() + '/' + new_id)

	else:

		return getattr(self, new_id)


class calendar_event(
	CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item):
	"""Represents an event."""

	meta_type = 'CalendarEvent'
	icon = 'misc_/CalendarFolder/dtmldoc.gif'

	meta_types = (
			{
				'name':'CalendarEventException',
				'action':'manage_add_calendar_event_exception_form',
			},
	)

	_properties = (
		{
			'id':'title',
			'type':'string'
		},
		{
			'id':'note',
			'type':'text'
		},
		{
			'id':'start_datetime',
			'type':'datetime'
		},
		{
			'id':'end_datetime',
			'type':'datetime'
		},
		{
			'id':'priority',
			'type':'int',
			'widget_type':'select',
			'data':'get_priority_options'
		},
		{
			'id':'status',
			'type':'int',
			'widget_type':'select',
			'data':'get_status_options',
		},
		{
			'id':'repeat_frequency',
			'type':'int'
		},
		{
			'id':'repeat_unit',
			'type':'int',
			'widget_type':'select',
			'data':'get_repeat_unit_options',
		},
		{
			'id':'repeat_days',
			'type':'list',
			'widget_type':'checkbox_collection',
			'data':'get_day_options',
		},
	)

	property_sheets = (
		('description',
			('title', 'note', 'priority', 'status'),
		),
		('times',
			('start_datetime', 'end_datetime'),
		),
		('re-occurrence',
			('repeat_frequency', 'repeat_unit', 'repeat_days'),
		),
	)

	__ac_permissions__ = (
		('View management screens',
			('manage_tabs', 'manage_main')
		),
		('Change Permissions',
			('manage_access',)
		),
		('View CalendarEvents',
			('',),
			['Owner', 'Manager', 'Calendar Manager', 'Calendar Editor', 'Calendar User'],
		),
	)


	def __init__(self, id):
		"Initialize the class instance"
		
		self.initialize_properties()
		self.id = id


	def start(self):
		"""Returns a string representing the start time of the event, fitted for 'snapping
		to the grid' of the TimetableTag."""

		# Breaking the law, breaking the law

		if self.REQUEST is not None:

			day = self.REQUEST['day']

		else:

			day = None

		# This is just a mess so we can snap into the grids of the TimetableTag

		if day is not None:

			if DateTime(self.start_datetime.Date()) < DateTime(day.Date()) and \
				self.repeat_unit == 0:

				hour = 0
				minute = 0

			else:

				minute = self.start_datetime.minute()
				hour = self.start_datetime.hour()

		else:

			minute = self.start_datetime.minute()
			hour = self.start_datetime.hour()

		if minute < 15:

			minute = 0

		elif minute < 45:

			minute = 30

		else:

			if hour == 23:

				minute = 59

			else:

				hour = hour + 1
				minute = 0

		return "%.2d:%.2d" % (hour, minute)

		
	def end(self):
		"""Returns a string representing the end time of the event, fitted for 'snapping
		to the grid' of the TimetableTag."""

		# Breaking the law, breaking the law		

		if self.REQUEST is not None:

			day = self.REQUEST['day']

		else:

			day = None

		# This is just a mess so we can snap into the grids of the TimetableTag

		if day is not None:

			if DateTime(self.end_datetime.Date()) > DateTime(day.Date()) and \
				self.repeat_unit == 0:

				hour = 23
				minute = 59

			else:

				minute = self.end_datetime.minute()
				hour = self.end_datetime.hour()

		else:

			minute = self.end_datetime.minute()
			hour = self.end_datetime.hour()

		if minute < 15:

			minute = 0

		elif minute < 45:

			minute = 30

		else:

			if hour == 23:

				minute = 59

			else:

				hour = hour + 1
				minute = 0

		return "%.2d:%.2d" % (hour, minute)


	def start_date(self):
		"""Returns the start date of the event."""

		return DateTime(self.start_datetime.Date())


	def end_date(self):
		"""Returns the end date of the event."""

		return DateTime(self.end_datetime.Date())


	def occupies_time(self, hour, day):
		"""Returns a truth value if the event occupies the given hour."""

		day = DateTime(day.Date())

		start_hour = 0
		start_minute = 0
		end_hour = 0
		end_minute = 0

		if self.repeat_unit == 0:

			if int(DateTime(self.start_datetime.Date())) != int(day):

				start_hour = 0

			else:

				start_hour = self.start_datetime.hour()

			if int(DateTime(self.end_datetime.Date())) != int(day):

				end_hour = 24

			else:

				end_hour = self.end_datetime.hour()

		else:

			start_hour = self.start_datetime.hour()
			end_hour = self.end_datetime.hour()

		start_minute = self.start_datetime.minute()
		end_minute = self.end_datetime.minute()

		lower_hour = start_hour
		higher_hour = lower_hour + 1

		if (start_minute > 29):

			time_start = higher_hour

		else:

			time_start = lower_hour

		lower_hour = end_hour
		higher_hour = lower_hour + 1

		if (end_minute < 30):

			time_end = lower_hour

		else:

			time_end = higher_hour
  
		if time_start == time_end:

			time_end = time_end + 1

		if time_start < hour and time_end > hour:

			ret = -1
		else: 

			if time_start > hour or time_end < hour + 1:

				ret = 0
			else:

				ret = 1

		return ret


	def occurs_on_day (self, day):
		"""Returns a truth value if self occurs on DateTime day."""

		start=self.start_date()
		end=self.end_date()
		day=DateTime(day.Date());

		# FIXME
		#if day in self.exceptions:
		#
		#	return 0

		if self.repeat_unit == 0: # No repeat unit

			if start <= day <= end:

				return 1

		if self.repeat_unit == 1: # Days

			if ((start <= day <= end) and
				(((start-day) % self.repeat_frequency)==0)):

				return 1

		if self.repeat_unit == 2: # Weeks
			day_weekstart=day-day.dow()
			event_weekstart=start-start.dow()

			# Why do we need to do an int conversion here?
			# And we need to circumvent the string casting somehow..

			if ((start <= day <= end) and
				((int(day_weekstart-event_weekstart) % (7*self.repeat_frequency))==0) and
				(str(day.dow()) in self.repeat_days)):

				return 1

		if self.repeat_unit == 3: # Months
			if ((start <= day <= end) and
				(((day.year()-start.year())*12+
				  (day.month()-start.month()))
				 % self.repeat_frequency==0) and
				(day.day() == start.day())):

				return 1

		if self.repeat_unit == 4: # Years
			if ((start <= day <= end) and
				(((start.year() - day.year()) %
				  self.repeat_frequency)==0) and
				(start.month() == day.month()) and
				(start.day() == day.day())):

				return 1

		return 0

default__class_init__(calendar_event)

