###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@thingamy.net>
#   Copyright (c) 2000,2001 Thingamy Ltd.
#   Copyright (c) 2001 James E. Blair <corvus@gnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.18 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import DTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import time
import calendar_event
from Products.WarpFramework import base_item, product_base_item
import todo_category_folder
import todo_category
from DateTime import DateTime
import calendar

manage_add_calendar_folder_form = DTMLFile('dtml/calendar_folder_add', globals())

class dummy:
	__allow_access_to_unprotected_subobjects__=1
	pass

def manage_add_calendar_folder(self, id, title,
			install_default_html='',
			install_default_theme='',
			install_default_id_generator='',
			use_as_shared_calendar='',
			REQUEST=None):
	"""Add an instance of the calendar_folder class."""

	self._setObject(id, calendar_folder(id, title,
				use_as_shared_calendar=use_as_shared_calendar,
				install_default_html=install_default_html,
				install_default_theme=install_default_theme,
				install_default_id_generator=install_default_id_generator))

	if REQUEST is not None:

		REQUEST.RESPONSE.redirect(self.absolute_url() + '/' + id)

	else:

		return getattr(self, id)


class calendar_folder(
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item,):
	"""Calendar used for scheduling events, todos and such."""

	index_html = DTMLFile('dtml/calendar_folder_index', globals())
	display = DTMLFile('dtml/calendar_folder_display', globals())
	calendar_day_navigation = DTMLFile('dtml/calendar_day_navigation', globals())

	meta_type = 'CalendarFolder'
	zbox_title = 'Calendar'
	product_name = meta_type

	_properties = (
		{'id':'title', 'type':'string'},
	)

	meta_types = (
		{
			'name':'CalendarEvent',
			'action':'manage_add_calendar_event'
		}
	)

	#manage_add_calendar_event_form = calendar_event.manage_add_calendar_event_form
	# Should we put the time checking logic at the event level?
	manage_add_calendar_event = calendar_event.manage_add_calendar_event

	manage_add_calendar_event = calendar_event.manage_add_calendar_event

	__ac_roles__ = ('Calendar Manager', 'Calendar Editor', 'Calendar User')

	__ac_permissions__ = (
		('View management screens',
			('manage_tabs', 'manage_main')
		),
		('Change Permissions',
			('manage_access',)
		),
		('View CalendarFolders',
			('', 'get_years', 'get_ymd', 'get_date', 'display_current_month',
				'get_today', 'display_current_day', 'display_next_day',
				'get_day', 'check_timespan', 'get_unique_id',
				'get_calendar'
			),
			['Manager', 'Owner', 'Calendar Manager', 'Calendar Editor', 'Calendar User'],
		),
		('Edit Calendar',
			('manage_add_calendar_event', 'manage_add_calendar_event_form'),
			['Manager', 'Owner', 'Calendar Manager', 'Calendar Editor'],
		),
		('Manage Calendar',
			('base_edit',),
			['Manager', 'Owner', 'Calendar Manager'],
		),
	)

	alternate_date_mappings = [
		{'name':'', 'title':'Select date'},
		{'name':1, 'title':'Today'},
		{'name':2, 'title':'Tomorrow'},
		{'name':7, 'title':'Next week'},
		{'name':31, 'title':'Next month'}
	]


	def format_hour (self, hour):
		if (hour==0): return '00:00'
		ret=str(hour)
		if(len(ret)==1): return '0'+ret+':00'
		return ret+':00'


	def get_status_options(self):
		"""Returns a selection of status options."""

		return (
			(0, 'Completed'),
			(1, 'Pending'),
			(2, 'Cancelled'),
		)


	def get_priority_options(self):
		"""Returns a selection of priority options."""

		return (
			(0, 'Low'),
			(1, 'Medium'),
			(2, 'High'),
		)


	def get_repeat_unit_options(self):
		"""Returns a selection of repeat unit options."""

		return (
			(0, 'Does not repeat'),
			(1, 'Days'),
			(2, 'Weeks'),
			# We just hide these for now
			#(3, 'Months'),
			#(4, 'Years'),
		)


	def get_day_options(self):
		"""Returns a selection of day options."""

		return (
			('1', 'Monday'),
			('2', 'Tuesday'),
			('3', 'Wednesday'),
			('4', 'Thursday'),
			('5', 'Friday'),
			('6', 'Saturday'),
			('0', 'Sunday')
		)


	def time_available(self, start, end):
		events=self.get_events_for_period(start, end)

		# we only have events on the date at this point.
		for event in events:

			if start <= event.start_datetime < end or \
				start > event.end_datetime >= end:

				return 0

		return 1


	def get_events_for_day(self, day):
		"""Returns the events for the given day."""

		start = day
		end = day + 1

		return self.get_events_for_period(start, end)


	def get_events_for_period(self, start, end):

		repeating_events_for_period_brains=self.Catalog(
			property_status=1,
			repeat_frequency=1,
			repeat_frequency_usage='range:min',
			start_datetime=end,
			start_datetime_usage='range:max',
			end_datetime=start,
			end_datetime_usage='range:min')

		repeating_events_for_period=[]

		for mybrain in repeating_events_for_period_brains:
			repeating_events_for_period.append(self.Catalog.getobject(mybrain.data_record_id_))

		repeating_events=[]

		for event in repeating_events_for_period:

			# This baby eats up a lot of time..

			for day in range(0, end - start):

				day = start + day

				if event.occurs_on_day(day):

					repeating_events.append(event)
					break

		events_for_period_brains=self.Catalog(
			property_status=1,
			repeat_frequency=0,
			start_datetime=end,
			start_datetime_usage='range:max',
			end_datetime=start,
			end_datetime_usage='range:min')

		events_for_period=[]

		for mybrain in events_for_period_brains:
			events_for_period.append(self.Catalog.getobject(mybrain.data_record_id_))

		return repeating_events + events_for_period

	
	def get_blocks (self, events, date):
		"""Returns time blocks for given events and date."""

		times=map (lambda x: None, range(0,24))
		blocks=[]
		for event in events:
			myblock=None
			start=event.start_datetime
			end=event.end_datetime
			for hour in range (0, 24):
				if (event.occupies_time(hour, date)):
					if (times[hour] == None): #nothing there, new block
						if (myblock == None):
							blocks.append([])
							myblock = len(blocks)-1
							blocks[myblock].append(event)
						times[hour]=myblock
					else: #something there, add to block
						if (myblock == None): #but this is our first appearance
							myblock = times[hour]
							blocks[myblock].append(event)
						else:
							# we already have a block, so we extend and absorb
							# the offending block.
							# unless the offending block is ours.
							current=0; oldblock=0
			  
							if (times[hour]!=myblock):
								blocks[myblock].extend(blocks[times[hour]])

								current=hour;
								oldblock=times[hour];
			  
								try:

									while (oldblock == times[current]):
										times[current] = myblock
										current=current+1

								except IndexError:

									# FIXME

									pass

		return (times, blocks)


	def get_previous_year(self, datetime):
		"""Returns the previous year."""

		previous_year = datetime.year() - 1

		return DateTime('%s/%s/%s' % (previous_year, datetime.month(), datetime.day()))


	def get_next_year(self, datetime):
		"""Returns the next year."""

		next_year = datetime.year() + 1

		return DateTime('%s/%s/%s' % (next_year, datetime.month(), datetime.day()))


	def old_manage_add_calendar_event(self, REQUEST):
		"""Add an instance of the calendar_event class."""

		start_datetime = REQUEST['start_datetime']
		end_datetime = REQUEST['end_datetime']

		if (not self.time_available(start_datetime,
	                end_datetime)):

			msg = "added_conflicting_event"

		else:

			msg = ""

		new_id = self.get_unique_id()
		title = REQUEST['title']
		note = REQUEST['note']
		priority = REQUEST['priority']
		status = REQUEST['status']
		repeat_frequency = REQUEST['repeat_frequency']
		repeat_unit = REQUEST['repeat_unit']

		if (hasattr(REQUEST, 'repeat_days')):

			repeat_days = REQUEST['repeat_days']

		else:

			repeat_days = []
	
		self._setObject(new_id, calendar_event.calendar_event(
					new_id))

		self[new_id].update_properties(title=title, note=note,
					start_datetime=start_datetime,
					end_datetime=end_datetime,
					priority=priority, status=status,
					repeat_frequency=repeat_frequency,
					repeat_unit=repeat_unit,
					repeat_days=repeat_days)

		REQUEST.RESPONSE.redirect(self.absolute_url() + '?%s' % msg)


	def __init__(self, id, title='',
			use_as_shared_calendar='',
			install_default_html='',
			install_default_theme='',
			install_default_id_generator=''):
		"Initialize the class instance"

		self.id = id
		self.title = title
		self.use_as_shared_calendar = use_as_shared_calendar
		self.snooze = 300

		from Products.ZCatalog.ZCatalog import ZCatalog

		self._setObject('Catalog', ZCatalog('Catalog', ''))

		self['Catalog'].manage_addIndex('start_datetime', 'FieldIndex')
		self['Catalog'].manage_addIndex('end_datetime', 'FieldIndex')
		self['Catalog'].manage_addIndex('repeat_frequency', 'FieldIndex')
		self['Catalog'].manage_addIndex('priority', 'FieldIndex')
		self['Catalog'].manage_addIndex('status', 'FieldIndex')
		self['Catalog'].manage_addIndex('repeat_unit', 'FieldIndex')
		self['Catalog'].manage_addIndex('repeat_days', 'FieldIndex')
		self['Catalog'].manage_addIndex('note', 'TextIndex')
		self['Catalog'].manage_addIndex('property_status', 'FieldIndex')

		product_base_item.utils.default_installation(
			self,
			install_default_html=install_default_html,
			install_default_theme=install_default_theme,
			install_default_id_generator=install_default_id_generator)

		todo_category_folder_id = todo_category_folder.manage_add_todo_category_folder(self,
						id='TodoCategories', title='Todo categories')

		todo_category.manage_add_todo_category(self[todo_category_folder_id],
								id='-2', title='Business')
		todo_category.manage_add_todo_category(self[todo_category_folder_id],
								id='-3', title='Personal')
		todo_category.manage_add_todo_category(self[todo_category_folder_id],
								id='-4', title='Recreational')


	def get_calendar(self):
		"""Returns the calendar folder instance."""

		return self


	def simple_navigation_html(self, day=None):
		"""Returns a simple month navigation list."""

		calendar_month_list = []
		calendar_months = [1,2,3,4,5,6,7,8,9,10,11,12]
		context_year, context_month, context_day = day.year(), day.month(), day.day()

		for month in calendar_months:

			month_html = """&nbsp;<a href="%s?day:date=%s">%s</a>&nbsp;""" % \
				(self.absolute_url(), DateTime("%s/%s/%s" % (context_year, month, context_day)), month)

			if month == context_month:

				is_current_month = 1

			else:

				is_current_month = None

			calendar_month_list.append({'html':month_html,'is_current_month':is_current_month})

		return calendar_month_list



	def get_month_structure(self, date):
		"""Returns a structure describing the days of the current month."""

		return calendar.monthcalendar(date.year(), date.month())


	def get_month_navigation(self, details=None, day=None):
		"""Returns an HTML formatted output of the current month."""

		tmp = """<table border="0" cellpadding="%s" cellspacing="%s"
		valign="top" width="100%%">""" % \
			(self.table_cellpadding0,
			self.table_cellspacing0)


		tmp = tmp + """
		<tr>
		<td valign="middle"> Mon </td>
		<td valign="middle"> Tue </td>
		<td valign="middle"> Wed </td>
		<td valign="middle"> Thu </td>
		<td valign="middle"> Fri </td>
		<td valign="middle"> Sat </td>
		<td valign="middle"> Sun </td>
		</tr>"""


		context_year, context_month, context_day = day.year(), day.month(), day.day()

		weeks = self.get_month_structure(day)

		for week_index in range(len(weeks)):

			if week_index % 2:

				bgcolor = self.color_sequence_even0

			else:

				bgcolor = self.color_sequence_odd0

			tmp = tmp + """<tr bgcolor="%s">""" % bgcolor

			week = weeks[week_index]
				
			for day in week:

				if day:

					# This will break now

					if details:

						object_count = len(self[utils.pad_integer(day)].objectIds())

						if object_count:

							formatted_object_count = " (%s)" % object_count

						else:

							formatted_object_count = ""

					else:

						formatted_object_count = ""

					tmp = tmp + \
						"""<td align="center"><a href="%s/?day:date=%s">%s</a></td>""" % \
						(
							self.absolute_url(),
							DateTime("%s/%s/%s" % (context_year, context_month, day)),
							day
						)

				else:

					tmp = tmp + """<td>&nbsp;</td>"""

			else:

				tmp = tmp + """</tr>"""

		tmp = tmp + """</table>"""

		return tmp


default__class_init__(calendar_folder)

