###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morphex $
# $Revision: 1.2 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from ZPublisher.HTTPRequest import html_quote
from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS
import OFS.Folder
import Products
from Products.WarpFramework import product_base_item, base_item
import mail_gateway
import mailbox
import thread
import cStringIO
import re
import virtual_message_container
import message_container
#import news_group_folder
import imap4_wrapper
import pop3_wrapper
import smtplib
import string
import rfc822_address_history
import DateTime
import importers
import ZODB

from Products import RFC822Message

gum_add_redirect = HTMLFile('dtml/gum_add_redirect', globals())
manage_add_gum_form = HTMLFile('dtml/gum_add', globals())


def manage_add_gum(self, id, local_username, email_address, title,
				local_password='local_password',
				local_protocol='pop3',
				mailhost='localhost',
				smtphost='localhost',
				install_default_html='',
				install_default_theme='',
				install_default_id_generator='',
				REQUEST=None, RESPONSE=None):
	"""Add an instance of the gum class."""

	self._setObject(id, gum(id, title, local_username, email_address,
				mailhost, smtphost,
				local_password,
				local_protocol,
				install_default_html=install_default_html,
				install_default_theme=install_default_theme,
				install_default_id_generator=install_default_id_generator))

	if REQUEST is not None:

		RESPONSE.redirect(self.absolute_url() + '/' + id)

	else:

		return self[id]


class gum(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	product_base_item.product_base_item,
	base_item.base_item
	):
	"""Product for handling email and news messages."""

	message_container_types = message_container.message_container.types

	meta_type = 'GUM'
	icon = 'misc_/GUM/Folder_icon.gif'
	default_catalog = 'Catalog'
	help_name = 'gum.stx'

	use_local_navigation = 0

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'local_username', 'type':'string'},
		{'id':'local_password', 'type':'string'},
		{'id':'local_protocol', 'type':'string'},
		{'id':'mailhost', 'type':'string'},
		{'id':'smtphost', 'type':'string'},
		{'id':'email_address', 'type':'string'},
		{'id':'default_newshost', 'type':'string'},
		{'id':'batch_size', 'type':'int'},
		{'id':'signature', 'type':'text'},
		{'id':'out_of_office', 'type':'boolean', 'data':'out_of_office_choices'},
		{'id':'out_of_office_message', 'type':'text'},
		)

	out_of_office_choices = \
			{
				# We do this to get the 'right' order in the edit menu.
				-1:'Yes',
				0:'No'
			}

	meta_types = (
		#{'name':'MailGateway',
		# 'action':'manage_add_mail_gateway_form'},
		{'name':'MessageContainer',
		 'action':'manage_add_message_container_form'},
		#{'name':'NewsGroupFolder',
		# 'action':'manage_add_news_group_folder_form'},
		)

	manage_add_mail_gateway_form = mail_gateway.manage_add_mail_gateway_form
	manage_add_mail_gateway = mail_gateway.manage_add_mail_gateway

	manage_add_message_container_form = message_container.manage_add_message_container_form
	manage_add_message_container = message_container.manage_add_message_container

	manage_add_virtual_message_container_form = \
		virtual_message_container.manage_add_virtual_message_container_form
	manage_add_virtual_message_container = \
		virtual_message_container.manage_add_virtual_message_container

	#manage_add_news_group_folder_form = news_group_folder.manage_add_news_group_folder_form
	#manage_add_news_group_folder = news_group_folder.manage_add_news_group_folder
	#news_group_folder_add_redirect = news_group_folder.news_group_folder_add_redirect

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add MailGateway',
		 ('manage_add_mail_gateway', 'manage_add_mail_gateway_form')),
		('Add MessageContainer',
		 ('manage_add_message_container', 'manage_add_message_container_form')),
		('Add VirtualMessageContainer',
		 ('manage_add_virtual_message_container', 'manage_add_virtual_message_container_form')),
		('View GUMs',
		 ('',)),
		)

	index_html = HTMLFile('dtml/gum_index', globals())
	base_edit = HTMLFile('dtml/gum_edit', globals())
	preferences = HTMLFile('dtml/gum_preferences', globals())
	advanced_preferences = HTMLFile('dtml/gum_advanced_preferences', globals())
	display = HTMLFile('dtml/gum_display', globals())
	find = HTMLFile('dtml/gum_find', globals())

	message_container_global_view = HTMLFile('dtml/gum_message_container_global_view', globals())
	message_container_global_view_unread = HTMLFile('dtml/gum_message_container_global_view_unread', globals())
	message_filter_priorities = HTMLFile('dtml/gum_message_filter_priorities', globals())
	import_messages = HTMLFile('dtml/gum_import_messages', globals())
	use_importer = HTMLFile('dtml/gum_use_importer', globals())

	product_name = 'GUM'


	def manage_afterAdd(self, item, container):
		"""Does some after adding magic."""

		from Products.ZCatalog.ZCatalog import ZCatalog
		self._setObject('Catalog', ZCatalog('Catalog', title='', container=self))
		self['Catalog'].manage_addIndex('message_container_id', 'FieldIndex')
		self['Catalog'].manage_addIndex('value', 'TextIndex')
		self['Catalog'].manage_addIndex('sender', 'FieldIndex')
		self['Catalog'].manage_addIndex('body', 'TextIndex')
		self['Catalog'].manage_addIndex('read', 'FieldIndex')
		self['Catalog'].manage_addIndex('content_type', 'TextIndex')
		self['Catalog'].manage_addIndex('filename', 'TextIndex')
		self['Catalog'].manage_addIndex('date_created', 'FieldIndex')
		self['Catalog'].manage_addIndex('type', 'FieldIndex')
		self['Catalog'].manage_addIndex('relative_url', 'FieldIndex')
		self['Catalog'].manage_addIndex('get_size', 'FieldIndex')

		self['Catalog'].manage_addColumn('read')
		self['Catalog'].manage_addColumn('sender')
		self['Catalog'].manage_addColumn('render_date')
		self['Catalog'].manage_addColumn('render_priority')
		self['Catalog'].manage_addColumn('render_size')
		self['Catalog'].manage_addColumn('render_html_subject')
		self['Catalog'].manage_addColumn('relative_url')		

		message_container.manage_add_message_container(self, 'Inbox',
							       'Incoming messages',
							       type=[0], id='Inbox')
		message_container.manage_add_message_container(self, 'Trash',
					       'Deleted messages',
					       type=[0], id='Trash')
		message_container.manage_add_message_container(self,
							       'Sent items',
							       'Outgoing messages',
							       type=[1], id='Sent')		
		message_container.manage_add_message_container(self, 'Drafts',
							       'Unfinished messages',
							       type=[2], id='Drafts')
		message_container.manage_add_message_container(self,
							       'Templates',
							       'Templates that save time',
							       type=[3], id='Templates')

	def __init__(self, id,
			title,
			local_username,
			email_address,
			mailhost,
			smtphost,
			local_password,
			local_protocol,
			signature='',
			message_retrieval_limit=100,
			install_default_html=None,
			install_default_theme=None,
			install_default_id_generator=None):
		"""Initialize the class instance."""

		self.id = id
		self.title = title

		self.local_username = local_username
		self.local_password = local_password
		self.local_protocol = local_protocol
		self.mailhost = mailhost
		self.smtphost = smtphost
		self.email_address = email_address
		self.signature = signature
		self.out_of_office = 0
		self.out_of_office_message = ''
		self.default_newshost = ''
		self.priority_count = 0
		self.notifications = []
		self.batch_size = 15
		self.message_retrieval_limit = message_retrieval_limit

		self.primary_incoming_folder = 'Inbox'
		self.primary_outgoing_folder = 'Sent'
		self.primary_trash_folder = 'Trash'
		self.primary_draft_folder = 'Drafts'
		self.primary_template_folder = 'Templates'

		rfc822_address_history.manage_add_rfc822_address_history(self,
					'IncomingHistory',
					'Your incoming address history')
		rfc822_address_history.manage_add_rfc822_address_history(self,
					'OutgoingHistory',
					'Your outgoing address history')

		product_base_item.utils.default_installation(
			self,
			install_default_html=install_default_html,
			install_default_theme=install_default_theme,
			install_default_id_generator=\
				install_default_id_generator
			)


	def get_mailbox_importers(self):
		"""Returns a list of message importers."""

		return importers.get_importers()


	def get_mailbox_importer(self, name):
		"""Returns the given importer."""

		return importers.get_importer(name)


	def import_mailbox(self, REQUEST, RESPONSE):
		"""Imports the mailbox."""

		file = REQUEST['file']
		storage_method = REQUEST['storage_method']
		importer = REQUEST['importer']

		importer = self.get_mailbox_importer(importer)

		import_session = importer.import_session(file)

		if storage_method == 'message_drop':

			message_filters = self.get_message_filters()

			while 1:

				_message = import_session.next()

				if _message is None:

					break

				_message.fp.seek(0)
				_message = RFC822Message.util.my_string_io(_message.fp.read())

				message = RFC822Message.rfc822_message_parse.rfc822_message_parse(
							_message)

				self.drop_message(message, message_filters=message_filters)

		else:

			raise NotImplemented, "The storage method '%s' is not implemented" % storage_method


	def get_preferences_url(self):
		"""Returns the preferences URL."""

		return self.relative_url() + '/preferences'


	def get_new_priority(self):
		"""Returns a new priority."""

		self.priority_count = self.priority_count + 1

		return self.priority_count


	def render_message_container_type(self, message_container_type):
		"""Renders the name of the message container type."""

		return self.message_container_types[int(message_container_type)]


	def render_message_container_title(self, message_container_id):
		"""Renders the name of the message container."""

		return self[message_container_id].title


	# FIXME
	# We need to add a primary folder selection
	# tool in the preferences menu


	def get_trash_folder(self):
		"""Returns the trash (deleted items) folder."""

		return self[self.primary_trash_folder]


	def get_primary_copy_to_folder(self):
		"""Returns the primary copy to folder."""

		return self.get_gum_message_containers(type=1)[0]


	def get_primary_incoming_folder(self):
		"""Returns the primary incoming folder."""

		try:

			return self[self.primary_incoming_folder]

		except KeyError:

			return self.get_gum_message_containers(type=0)[0]


	def get_primary_outgoing_folder(self):
		"""Returns the primary outgoing folder."""

		try:

			return self[self.primary_outgoing_folder]

		except KeyError:

			return self.get_gum_message_containers(type=1)[0]


	def get_primary_draft_folder(self):
		"""Returns the primary draft folder."""

		try:

			return self[self.primary_draft_folder]

		except KeyError:

			return self.get_gum_message_containers(type=2)[0]


	def get_primary_template_folder(self):
		"""Returns the template folder."""

		try:

			return self[self.primary_template_folder]

		except KeyError:

			return self.get_gum_message_containers(type=3)[0]


	def get_gum(self):
		"""Returns self."""

		return self


	def get_mail_gateway(self):
		"""Returns default mail gateway."""

		try:

			return self[self.default_mail_gateway]

		except AttributeError:

			return self.objectValues('MailGateway')[0]


	def get_news_host(self):
		"""Returns a newshost."""

		# FIXME -- this is not (does not feel like) the right
		# way to do it..

		news_hosts = []

		reference = self

		while 1:

			try:

				news_hosts = reference.objectValues('NewsHost')

			# We've come to the end of the line

			except AttributeError:

				break

			if news_hosts:

				break

			reference = reference.getParentNode()

		if not news_hosts:

			raise 'NoNewsHosts', 'No news hosts are configured on this server'

		return news_hosts[0]


	def get_gum_version(self):
		"""Returns a version number."""

		return open(SOFTWARE_HOME + '/Products/' + self.product_name \
					+ '/version.txt').read()


	def get_gum_website(self):
		"""Returns the GUM website."""

		return open(SOFTWARE_HOME + '/Products/' + self.product_name \
				+ '/website.txt').read()


	def get_number_of_mail_messages(self):
		"""Gets the number of unread messages in all incoming mailboxes."""

		count = 0

		for message_container in \
				self.get_gum_message_containers(type=0):

			count = count + message_container.get_number_of_messages()

		return count


	def get_number_of_unread_mail_messages(self):
		"""Gets the number of unread messages in all mailboxes."""

		# To see where the time is spent
		# FIXME

		return self.get_number_of_mail_messages()

		count = 0

		for message_container in self.objectValues('MessageContainer'):

			count = count + message_container.get_number_of_unread_messages()

		return count


	def get_message_by_id(self, id):
		"""Returns a messages based on its id."""

		# ?FIXME?

		return self.unrestrictedTraverse(self.Catalog.getpath(self.Catalog.searchResults(id=id)[0].getRID()))


	def get_message_filters(self):
		"""Returns all message filters sorted by priority."""


		message_filters = []
		catalog = self.Catalog

		for catalog_brain in catalog.searchResults(
			meta_type='MessageFilter',
			):

			message_filters.append(self.unrestrictedTraverse(
				catalog.getpath(catalog_brain.getRID())))

		message_filters.sort(lambda x,y: cmp(x.priority, y.priority))

		return message_filters


	def render_message_container_type_widget(self, REQUEST):
		"""Renders an HTML widget."""

		message_container_type = int(REQUEST['message_container_type'])
		select_widget = """<select name="message_container_type:int">"""

		for option in self.message_container_types.keys():

			if message_container_type == option:

				selected = " selected"

			else:

				selected = ""

			select_widget = select_widget + """<option value="%s"%s>%s</option>""" % \
						(option, selected, self.message_container_types[option])

		return select_widget + "</select>"


	def get_incoming_message_containers(self):
		"""Returns incoming message containers."""

		return self.get_gum_message_containers(type=0)


	def get_outgoing_message_containers(self):
		"""Returns outgoing message containers."""

		return self.get_gum_message_containers(type=1)


	def get_draft_message_containers(self):
		"""Returns draft message containers."""

		return self.get_gum_message_containers(type=2)


	def get_template_message_containers(self):
		"""Returns templatet message containers."""

		return self.get_gum_message_containers(type=3)


	def retrieve_messages(self, REQUEST=None, RESPONSE=None,
				run_as_thread=0, number_of_messages=None):
		"""Wrapper so we can choose whether to retrieve messages in threaded mode."""

		self._retrieve_messages(self, REQUEST=REQUEST,
					RESPONSE=RESPONSE,
					number_of_messages=number_of_messages)

		RESPONSE.redirect(self.relative_url() + '/')
	

	def retrieve_messages(self, REQUEST=None, RESPONSE=None,
			number_of_messages=None):
		"""Retrieves messages from the local server."""

		if number_of_messages is None:

			number_of_messages = self.message_retrieval_limit

		message_filters = self.get_message_filters()

		primary_folder = self.get_primary_incoming_folder()

		if self.local_username and \
			self.local_password and self.local_protocol:

			if self.local_protocol == 'imap':

				raise NotImplemented

			elif self.local_protocol == 'pop3':

				session = pop3_wrapper.pop3_session(self.mailhost)

			else:

				raise 'UnknownProtocolException', self.local_protocol

			session.login(self.local_username, self.local_password)

			mbox = session

		else:

			# FIXME -- poor hack
			# Move to updating of properties?

			checkname = re.compile(r'^([A-Z]|[a-z]|[0-9])*')

			if not checkname.search(self.local_username):

				raise 'NoWay', 'You\'re not getting away with this username.. %s'\
						% self.local_username

			mbox = mailbox.UnixMailbox(
				open('/var/spool/mail/'+self.local_username))

		message_count = 1

		while 1:

			if message_count > number_of_messages:

				break

			if not message_count % 5:

				if RESPONSE is not None:

					RESPONSE.write('5')

				if not message_count % 100:

					RESPONSE.write('\n')

				RESPONSE.flush()

			_message = mbox.next()

			_message.fp.seek(0)
			_message = RFC822Message.util.my_string_io(_message.fp.read())

			message = RFC822Message.rfc822_message_parse.rfc822_message_parse(
				_message)

			self.drop_message(message, message_filters=message_filters)

			message_count = message_count + 1

			RESPONSE.write('*')

		try:

			session.quit()

		except AttributeError:

			pass

		return 'success'


	def compose_mail(self, sender='', subject='', body='', recipients=[],
				newsgroups=[], headers={}, REQUEST=None, RESPONSE=None):
		"""Creates a new instance of a message and redirects to it."""

		subject = subject or ''
		body = body or ''
		recipients = recipients or []
		newsgroups = newsgroups or []
		# FIXME -- add handling of headers

		if string.strip(self.signature):

			signature_append = '\n--\n' + self.signature

		else:

			signature_append = ''


		if REQUEST:

			if REQUEST.has_key('subject') and REQUEST['subject']:

				subject = REQUEST['subject']

			if REQUEST.has_key('body') and REQUEST['body']:

				body = REQUEST['body'] + signature_append

			else:

				body = body + signature_append

			if REQUEST.has_key('recipients') and REQUEST['recipients']:

				recipients = REQUEST['recipients']

				if type(recipients) == type(''):

					recipients = [recipients]

			if REQUEST.has_key('newsgroups') and REQUEST['newsgroups']:

				newsgroups = REQUEST['newsgroups']

				if type(newsgroups) == type(''):

					newsgroups = [newsgroups]


		draft_folder = self.get_primary_draft_folder()

		new_id = str(self.get_unique_id())

		RFC822Message.rfc822_message.manage_add_rfc822_message(draft_folder,
								new_id,
								sender=self.email_address,
								body=body,
								subject=subject)

		message_reference = draft_folder[new_id]

		if sender:

			message_reference.sender = sender

		message_reference.set_header('copy-to-folder',
				self.get_primary_copy_to_folder().relative_url())

		if recipients:

			for recipient in recipients:

				new_id = str(self.get_unique_id())

				message_reference['to'].manage_add_rfc822_address(
									new_id,
									recipient)

		if newsgroups:

			for newsgroup in newsgroups:

				message_reference.add_newsgroup(newsgroup)


		if RESPONSE:

			RESPONSE.redirect(message_reference.absolute_url())

		else:

			return message_reference


	def get_gum_message_containers(self, type=None):
		"""Returns a list of all available top level message containers."""

		message_containers = []
		catalog = self.Catalog

		for catalog_brain in catalog.searchResults(
			meta_type='MessageContainer',
			type=type,
			message_container_id=self.id):

			message_containers.append(self.unrestrictedTraverse(
				catalog.getpath(catalog_brain.getRID())))

		return message_containers


	def get_all_gum_message_containers(self, type=None):
		"""Returns a list of all available message containers of the given type."""

		message_containers = []
		catalog = self.Catalog

		for catalog_brain in catalog.searchResults(
			meta_type='MessageContainer',
			type=type):

			message_containers.append(self.unrestrictedTraverse(
				catalog.getpath(catalog_brain.getRID())))

		message_containers.sort(lambda x, y: cmp(x.relative_url(), y.relative_url()))

		return message_containers



	def get_newsgroup_folders(self):
		"""Returns a list of all the message containers."""

		return self.objectValues('NewsGroupFolder')


	def manage_preferences(self, REQUEST, RESPONSE):
		"""Manages preferences."""

		self.base_edit_properties(REQUEST, RESPONSE)

		RESPONSE.redirect(self.relative_url() + '/preferences')


	def get_all_messages_in_container_type(self, type):
		"""Returns all messages in given container type."""

		type = int(type)
		message_containers = []
		messages = []
		catalog = self.Catalog

		for message_container in self.get_all_gum_message_containers(type=type):

			for message in catalog.searchResults(
				meta_type='RFC822Message',
				message_container_id=message_container.id):

				messages.append(message) 

		return messages


	def get_all_available_newshosts(self):
		"""Returns a list of all available news_host objects."""

		news_hosts = []

		reference = self

		while 1:

			try:

				_news_hosts = reference.objectValues('NewsHost')

			except AttributeError:

				# We've come to the end of the road..

				break

			if _news_hosts:

				news_hosts.extend(_news_hosts)

			reference = reference.getParentNode()

		return news_hosts


	def move_message_filter_up(self, REQUEST, RESPONSE):
		"""Increases the priority of given message filter."""

		priority = int(REQUEST['priority'])
		message_filters = self.get_message_filters()

		for message_filter_index in range(len(message_filters)):

			message_filter = message_filters[message_filter_index]

			if message_filter.priority == priority:

				downgraded_filter = message_filters[message_filter_index - 1]

				new_priority = downgraded_filter.priority
				message_filter.priority = new_priority
				downgraded_filter.priority = priority

				break

		RESPONSE.redirect(self.absolute_url() + '/message_filter_priorities')


	def move_message_filter_down(self, REQUEST, RESPONSE):
		"""Decreases the priority of given message filter."""

		priority = int(REQUEST['priority'])
		message_filters = self.get_message_filters()

		for message_filter_index in range(len(message_filters)):

			message_filter = message_filters[message_filter_index]

			if message_filter.priority == priority:

				try:

					upgraded_filter = message_filters[message_filter_index + 1]

				except IndexError:

					upgraded_filter = message_filters[0]

				new_priority = upgraded_filter.priority
				message_filter.priority = new_priority
				upgraded_filter.priority = priority

				break


		RESPONSE.redirect(self.absolute_url() + '/message_filter_priorities')


	def get_outgoing_address_history(self):
		"""Returns the outgoing address history."""

		return self.OutgoingHistory


	def get_incoming_address_history(self):
		"""Returns the incoming address history."""

		return self.IncomingHistory
	

	def drop_outgoing_address(self, email, realname='', comment=''):
		"""Drops an address into the outgoing address history object."""

		new_id = self.get_unique_id()

		history = self.get_outgoing_address_history()

		# FIXME -- what's ut with TypeErrors on the comment keyword?

		history.manage_add_rfc822_address(new_id, email)

		try:

			global_outgoing_history = self.get_global_outgoing_history()
			new_id = self.get_unique_id()
			global_outgoing_history.manage_add_rfc822_address(new_id, email)

		except AttributeError:

			pass
		

	def drop_incoming_address(self, email, realname='', comment=''):
		"""Drops an address into the incmoing address history object."""

		new_id = self.get_unique_id()

		history = self.get_incoming_address_history()

		# FIXME -- what's up with TypeErrors on the comment keyword?

		history.manage_add_rfc822_address(new_id, email)


		try:

			global_incoming_history = self.get_global_incoming_history()
			new_id = self.get_unique_id()
			global_incoming_history.manage_add_rfc822_address(new_id, email)

		except AttributeError:

			pass


	def drop_message(self, message, message_filters=None):
		"""Drops a message to a message container with a matching mail filter,
		or alternatively to the default incoming folder."""

		try:

			for recipient in [message.sender] + message.to + message.cc:

				if not recipient[-1] == self.email_address:

					self.drop_incoming_address(email=recipient[-1], realname=recipient[0])

		except Exception, value:

			import sys, traceback, string
			type, val, tb = sys.exc_info()
			sys.stderr.write(string.join(traceback.format_exception(type, val, tb), ''))
			del type, val, tb

		message_filters = message_filters or self.get_message_filters()

		try:

			if self.out_of_office:

				message_reference = self.compose_mail(subject='Currently out of office',
								body=self.out_of_office_message,
								recipients=[message.sender[-1]])
				message_reference.headers['copy-to-folder'] = ''
				message_reference.headers = message_reference.headers

				message_reference.send()

		# FIXME

		except Exception, value:

			import sys, traceback, string
			type, val, tb = sys.exc_info()
			sys.stderr.write(string.join(traceback.format_exception(type, val, tb), ''))
			del type, val, tb

		for message_filter in message_filters:

			if message_filter.match(message.headers):

				RFC822Message.util.store_message(
					message_filter.getParentNode(),
					message,
					readonly=1)

				break

		else:

			RFC822Message.util.store_message(
				self.get_primary_incoming_folder(),
				message, readonly=1)

		try:

			subject = message.headers['subject']

		except KeyError:

			subject = '[No subject]'

		self.add_notification(
			'A new message has arrived from %s with subject %s' % (message.sender[1], html_quote(subject)))


	def add_notification(self, message):
		"""Adds a message to the notification queue."""

		try:

			self.getParentNode().add_notification(message)

		except AttributeError:

			self.notifications.append(message)


	def get_notifications(self, reset=None):
		"""Returns a list of notify messages."""

		notifications = self.notifications

		if reset:

			self.reset_notifications()

		return notifications


	def reset_notifications(self):
		"""Resets the notify list."""

		self.notifications = []


	def build_smtp(self):
		"""Returns an SMTP connection to the default SMTP host."""

		return smtplib.SMTP(self.smtphost)


	def post(self, message, raw_message=None):
		"""Deconstructs an rfc822_message object and posts it."""

		smtp_connection = self.build_smtp()

		if raw_message is None:

			raw_message = message.raw_message()

		smtp_connection.sendmail(message.sender, message.get_recipients(),
						raw_message)

		smtp_connection.quit()


default__class_init__(gum)
