###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.11 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import news_group_storage
from Products.WarpFramework import base_item, product_base_item
import nntplib
import string
import DateTime
from Acquisition import aq_base

debug = 1

news_host_add_redirect = HTMLFile('dtml/news_host_add_redirect', globals())
manage_add_news_host_form = HTMLFile('dtml/news_host_add', globals())


def manage_add_news_host(self, id='', server='', port='',
				install_default_html='',
				install_default_theme='',
				install_default_id_generator='',
				REQUEST=None, RESPONSE=None):
	"""Add an instance of the news_host class."""

	if not id:

		id = str(self.get_unique_id())

	self._setObject(id, news_host(id, server, port,
				install_default_html=install_default_html,
				install_default_theme=install_default_theme,
				install_default_id_generator=install_default_id_generator))

	if RESPONSE is not None:

		RESPONSE.redirect(self.absolute_url())

	else:

		return self[id]


class news_host(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	Products.BTreeFolder.BTreeFolder.BTreeFolder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	product_base_item.product_base_item,
	base_item.base_item
	):
	"Documentation string"

	product_name = 'NewsHost'

	meta_type = 'NewsHost'
	icon = 'misc_/NewsHost/Folder_icon.gif'

	_properties = (
		{'id':'server', 'type':'string'},
		{'id':'port', 'type':'int'},
		{'id':'active_list', 'type':'lines'},
		)

	meta_types = (
		{'name':'NewsGroupStorage',
		 'action':'manage_add_news_group_storage_form'},
		)

	manage_add_news_group_storage_form = news_group_storage.manage_add_news_group_storage_form
	manage_add_news_group_storage = news_group_storage.manage_add_news_group_storage
	news_group_storage_add_redirect = news_group_storage.news_group_storage_add_redirect

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add NewsGroupStorage',
		 ('manage_add_news_group_storage',
		  'manage_add_news_group_storage_form')),
		('View NewsHosts',
		 ('',)),
		)

	index_html = HTMLFile('dtml/news_host_index', globals())
	active_list_index = HTMLFile('dtml/active_list_index', globals())
	base_edit = HTMLFile('dtml/news_host_edit', globals())
	display = HTMLFile('dtml/news_host_display', globals())


	def __init__(self, id, server='', port=None, active_list=[],
			install_default_html=None,
			install_default_theme=None,
			install_default_id_generator=None):
		"""Initialize the class instance."""

		Products.BTreeFolder.BTreeFolder.BTreeFolder.__init__(self)

		from Products.ZCatalog.ZCatalog import ZCatalog
		self._setObject('Catalog', ZCatalog('Catalog', title='', container=self))

		self['Catalog'].manage_addIndex('message_container_id', 'FieldIndex')
		self['Catalog'].manage_addIndex('value', 'TextIndex')
		self['Catalog'].manage_addIndex('sender', 'FieldIndex')
		self['Catalog'].manage_addIndex('body', 'TextIndex')
		self['Catalog'].manage_addIndex('read', 'FieldIndex')
		self['Catalog'].manage_addIndex('data', 'TextIndex')
		self['Catalog'].manage_addIndex('content_type', 'TextIndex')
		self['Catalog'].manage_addIndex('filename', 'TextIndex')
		self['Catalog'].manage_addIndex('date_created', 'FieldIndex')

		self.id = id
		self.server = server
		self.port = port
		self.active_list = active_list
		self.newgroups_timestamp = 0

		
		
		product_base_item.utils.default_installation(
			self,
			install_default_html=install_default_html,
			install_default_theme=install_default_theme,
			install_default_id_generator=install_default_id_generator
				)

		connection = self.refresh()


	def manage_update_newgroups(self, RESPONSE):
		"""HTTP wrapper for update_activelist."""

		RESPONSE.write("""<html><head><title>Updating active list</title></head>""")
		RESPONSE.write("""<body>Please wait while the active list gets updated...<br>""")
		RESPONSE.flush()

		success, server_response = self.update_activelist()

		if success:

			RESPONSE.write("""Active list updated!<br>""")

		else:

			RESPONSE.write("""Active list was not updated, the server said: %s""" % server_response)

		RESPONSE.write("""<form method="post"><input type="submit" value=" Go back " name="%s/base_edit:method"></form>""" % self.relative_url())
		RESPONSE.write("""</body></html>""")

		#RESPONSE.redirect(self.relative_url() + '/base_edit')


	def update_activelist(self):
		"""Updates the active_list with new newsgroups."""

		connection = self.refresh()
		current_timestamp = DateTime.DateTime(self.newgroups_timestamp)
		new_timestamp = DateTime.DateTime()

		server_response, newgroups = connection.newgroups(
			current_timestamp.strftime("%y%m%d"),
			string.join(string.split(
					current_timestamp.Time(), ':'), ''))

		for newsgroup in  map(lambda x: string.split(x)[0], newgroups):

			self.add_newsgroup(newsgroup)

		self.newgroups_timestamp = int(new_timestamp)

		if server_response[:3] != '231':

			return None, server_response

		else:

			return 1, server_response


	def add_newsgroup(self, newsgroup):
		"""Adds a newsgroup."""

		path = string.split(newsgroup, '.')

		reference = self

		for step in path:

			try:

				getattr(reference.aq_base, step)
				reference = getattr(reference, step)

			except AttributeError:

				_reference = reference.manage_add_news_group_storage(step)
				reference = _reference
				reference.is_newsgroup = 0
				reference.index_object()

		reference.is_newsgroup = 1
		reference.index_object()


	def refresh(self):
		"""Opens a connection."""

		if self.port:

			return nntplib.NNTP(self.server, self.port)

		else:

			return nntplib.NNTP(self.server)


	def get_subscribed_newsgroups(self):
		"""Returns a list of subscribed newsgroups."""

		subscribed_newsgroups = []
		catalog = self.Catalog

		for catalog_brain in catalog.searchResults(
			meta_type='NewsGroupStorage',
			is_newsgroup=1):

			subscribed_newsgroups.append(self.unrestrictedTraverse(
				catalog.getpath(catalog_brain.getRID())))

		return subscribed_newsgroups


	def retrieve_group(self, newsgroup, run_as_thread=1, RESPONSE=None):
		"""Retrieves messages for a given group."""

		self._retrieve_group(newsgroup, RESPONSE)
		return


	def get_number_of_messages(self):
		"""Returns the number of messages contained in the
		news group storages."""

		return len(self.Catalog.searchResults(
			meta_type='RFC822Message'))


	def _retrieve_group(self, newsgroup, RESPONSE):
		"""Retrieves messages for a given group."""

		connection = self.refresh()

		news_group_storage = self[newsgroup]

		dummy, dummy, first, last, dummy = connection.group(newsgroup)

		first = int(first)
		last = int(last)


		# If there are new messages.

		if last > news_group_storage.last_retrieved_message:

			# If the first (available) message on the newsserver
			# is more recent than then last message we retrieved
			# we can safely skip our own counter and start retrieving
			# based upon the message number supplied by the server.

			if first > news_group_storage.last_retrieved_message:

				message_retrieval_start = first

			else:

				message_retrieval_start = news_group_storage.last_retrieved_message

			for message_number in range(message_retrieval_start,
							last + 1):

				try:
					
					# Should we add more program logic here, WRT to
					# status codes, etc?

					article = string.join(
						connection.article(str(message_number))[-1],'\n')

					Products.RFC822Message.util.store_unparsed_message(
						news_group_storage, article, readonly=1)

					news_group_storage.last_retrieved_message = message_number

					if RESPONSE is not None:

						if not (message_number - message_retrieval_start) % 50:

							RESPONSE.write('<br>&nbsp;&nbsp;')

						RESPONSE.write('.')
						RESPONSE.flush()

					else:

						raise str(RESPONSE)

				except nntplib.error_temp, value:

					# Add program logic (handling of status, etc)
					# FIXME

					pass


		else:

			# No new messages for given group.

			pass


	def retrieve(self, REQUEST, RESPONSE):
		"""The actual retrieval.."""

		RESPONSE.write("""
<html>
<head>
	<title>Progress for news retrieval</title>
</head>
<body>
""")

		for newsgroup in self.get_subscribed_newsgroups():

			RESPONSE.write("""Retrieving %s<br>&nbsp;&nbsp;""" % newsgroup.id)

			self.retrieve_group(newsgroup.id, RESPONSE=RESPONSE)

			if RESPONSE is not None:

				# We've retrieved the new messages, if any, from the newsgroup

				RESPONSE.write('&nbsp;&nbsp;Finished!<br>')

				RESPONSE.flush()

		else:

			RESPONSE.write('</body></html>')

		RESPONSE.flush()


	def post(self, message, raw_message=None):
		"""Posts a message."""

		connection = self.refresh()

		if raw_message is None:

			raw_message = message.raw_message()

		return connection.post(
			Products.RFC822Message.util.my_string_io(raw_message))


default__class_init__(news_host)